using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration.Install;
using System.ServiceProcess;
using HumanityServices;

namespace HumanityServices
{
	/// <summary>
	/// Windows Service Installer: ProjectInstaller
	/// 2005 Advance Guard Technologies - All Rights reserved
	/// Created: 2005.05.06 by ken norwick
	/// This installer class MUST derive from: (System.Configuration.Install.Installer)
	/// It installs and starts the specified Windows Service
	/// </summary>
	[RunInstaller(true)]
	public class ProjectInstaller : System.Configuration.Install.Installer
	{
		private System.ServiceProcess.ServiceProcessInstaller serviceProcessInstaller1;
		private System.ServiceProcess.ServiceInstaller serviceInstaller1;

		/// <summary>
		/// 2005 Advance Guard Technologies - All Rights reserved
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// Constructor: ProjectInstaller
		/// 2005 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.04.06 by ken norwick
		/// Main entry point for the service project installer
		/// </summary>
		public ProjectInstaller()
		{
			// This call is required by the Designer.
			InitializeComponent();
			
		}

		/// <summary>
		/// Method: Dispose (overridden from base class)
		/// 2005 Advance Guard Technologies - All Rights reserved
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// EventHandler: After_Install()
		/// 2005 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This function processes the post installation event activities for
		/// the windows service, by starting the service if it is not already
		/// running. This allows the background service to get to work right
		/// away making sure that the portals are ready for their first use.
		/// Note: You must register this eventhandler in the InitializeComponent
		/// method for this to be active. Important !!!
		/// </summary>
		/// <param name="sender">object</param>
		/// <param name="e">InstallEventArgs - any arguments required by the handler</param>
		private void After_Install(object sender, InstallEventArgs e)
		{
			// Add steps to perform any actions after the install process.
			try
			{
				//start the windows service after installation...
				ServiceController service = new ServiceController("WebRefresher");
				if(service != null)
				{
					if ( service.Status != ServiceControllerStatus.Running )
					{
						service.Start();
					
					}
				}
			}
			catch
			{

			}
		}//end of method


		/// <summary>
		/// Method: Install
		/// Override the base Install methos so that we can modify the service description
		/// </summary>
		/// <param name="stateServer"></param>
		public override void Install(IDictionary stateServer)
		{
			Microsoft.Win32.RegistryKey system,
				//HKEY_LOCAL_MACHINE\Services\CurrentControlSet
				currentControlSet,
				//...\Services
				services,
				//...\<Service Name>
				service,
				//...\Parameters - this is where you can put service-specific configuration
				config; 

			try
			{
				//Let the project installer do its job
				base.Install(stateServer);

				//Open the HKEY_LOCAL_MACHINE\SYSTEM key
				system = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("System");
				//Open CurrentControlSet
				currentControlSet = system.OpenSubKey("CurrentControlSet");
				//Go to the services key
				services = currentControlSet.OpenSubKey("Services");
				//Open the key for your service, and allow writing
				service = services.OpenSubKey(this.serviceInstaller1.ServiceName, true);
				//Add your service's description as a REG_SZ value named "Description"
				service.SetValue("Description", "Periodically refreshes web sites to the IIS cache.  ");
				//(Optional) Add some custom information your service will use...
				config = service.CreateSubKey("Parameters");
			}
			catch(Exception e)
			{
				Console.WriteLine("An exception was thrown during service installation:\n" + e.ToString());
			}
		}

		/// <summary>
		/// Method: Uninstall (overridden)
		/// </summary>
		/// <param name="stateServer"></param>
		public override void Uninstall(IDictionary stateServer)
		{
			Microsoft.Win32.RegistryKey system,
				currentControlSet,
				services,
				service;

			try
			{
				//Drill down to the service key and open it with write permission
				system = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("System");
				currentControlSet = system.OpenSubKey("CurrentControlSet");
				services = currentControlSet.OpenSubKey("Services");
				service = services.OpenSubKey(this.serviceInstaller1.ServiceName, true);
				//Delete any keys you created during installation (or that your service created)
				service.DeleteSubKeyTree("Parameters");
				//...
			}
			catch(Exception e)
			{
				Console.WriteLine("Exception encountered while uninstalling service:\n" + e.ToString());
			}
			finally
			{
				//Let the project installer do its job
				base.Uninstall(stateServer);
			}
		}


		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.serviceProcessInstaller1 = new System.ServiceProcess.ServiceProcessInstaller();
			this.serviceInstaller1 = new System.ServiceProcess.ServiceInstaller();
			// 
			// serviceProcessInstaller1
			// 
			this.serviceProcessInstaller1.Account = System.ServiceProcess.ServiceAccount.LocalSystem;
			this.serviceProcessInstaller1.Password = null;
			this.serviceProcessInstaller1.Username = null;
			// 
			// serviceInstaller1
			// 
			this.serviceInstaller1.DisplayName = "HumanityTHUNDER Web Refresh Service";
			this.serviceInstaller1.ServiceName = "WebRefresher";
			this.serviceInstaller1.StartType = System.ServiceProcess.ServiceStartMode.Automatic;
			this.serviceInstaller1.AfterInstall += new System.Configuration.Install.InstallEventHandler(this.After_Install);
			// 
			// ProjectInstaller
			// 
			this.Installers.AddRange(new System.Configuration.Install.Installer[] {
																					  this.serviceProcessInstaller1,
																					  this.serviceInstaller1});

		}
		#endregion
	}
}
